@@ -14,27 +14,24 @@ import webbrowser
 from datetime import datetime
 from pathlib import Path
 from queue import Queue
-
-# from multiprocessing import Process, Queue
-from threading import Thread
-from typing import Literal
+from typing import List, Literal
 from urllib.parse import ParseResult, urlparse
 
-import procrunner
 import requests
 
+# from multiprocessing import Process, Queue
+from rich.prompt import Confirm
+
 import murfey.client.rsync
 import murfey.client.update
 import murfey.client.watchdir
 import murfey.client.websocket
-from murfey.client.analyser import Analyser
 from murfey.client.customlogging import CustomHandler, DirectableRichHandler
 from murfey.client.gain_ref import determine_gain_ref
 from murfey.client.instance_environment import MurfeyInstanceEnvironment
-from murfey.client.rsync import RSyncer
 from murfey.client.tui.app import MurfeyTUI
 from murfey.client.tui.status_bar import StatusBar
-from murfey.util.models import Visit
+from murfey.util import _get_visit_list
 
 # from asyncio import Queue
 
@@ -78,14 +75,6 @@ def _check_for_updates(
         print(f"Murfey update check failed with {e}")
 
 
-def _get_visit_list(api_base: ParseResult, demo: bool = False):
-    get_visits_url = api_base._replace(path="/visits_raw")
-    server_reply = requests.get(get_visits_url.geturl())
-    if server_reply.status_code != 200:
-        raise ValueError(f"Server unreachable ({server_reply.status_code})")
-    return [Visit.parse_obj(v) for v in server_reply.json()]
-
-
 def run():
     config = read_config()
     try:
@@ -133,24 +122,24 @@ def run():
         action="store_true",
     )
     parser.add_argument(
-        "--appearance_time",
+        "--appearance-time",
         type=float,
         default=-1,
         help="Only consider top level directories that have appeared more recently than this many hours ago",
     )
     parser.add_argument(
-        "--real_dc",
+        "--fake-dc",
         action="store_true",
         default=False,
-        help="Actually perform data collection related calls to API (will do inserts in ISPyB)",
+        help="Do not perform data collection related calls to API (avoids database inserts)",
     )
     parser.add_argument(
-        "--transfer_all",
+        "--time-based-transfer",
         action="store_true",
-        help="Transfer all files in current directory regardless of age",
+        help="Transfer new files",
     )
     parser.add_argument(
-        "--no_transfer",
+        "--no-transfer",
         action="store_true",
         help="Avoid actually transferring files",
     )
@@ -166,10 +155,22 @@ def run():
         help="Perform rsync transfers locally rather than remotely",
     )
     parser.add_argument(
-        "--force_mdoc_metadata",
+        "--ignore-mdoc-metadata",
         action="store_true",
         default=False,
-        help="Force metadata to be read from mdoc files",
+        help="Do not attempt to read metadata from all mdoc files",
+    )
+    parser.add_argument(
+        "--remove-files",
+        action="store_true",
+        default=False,
+        help="Remove source files immediately after their transfer",
+    )
+    parser.add_argument(
+        "--relax",
+        action="store_true",
+        default=False,
+        help="Relax the condition that the source directory needs to be recognised from the configuration",
     )
 
     args = parser.parse_args()
@@ -181,6 +182,13 @@ def run():
             exit("Unknown server protocol. Only http:// and https:// are allowed")
         args.server = f"http://{args.server}"
 
+    if args.remove_files:
+        remove_prompt = Confirm.ask(
+            f"Are you sure you want to remove files from {args.source or Path('.').resolve()}?"
+        )
+        if not remove_prompt:
+            exit("Exiting")
+
     murfey_url = urlparse(args.server, allow_fragments=False)
     if args.server != known_server:
         # New server specified. Verify that it is real
@@ -214,7 +222,7 @@ def run():
                 break
     if not ongoing_visits:
         print("Ongoing visits:")
-        ongoing_visits = _get_visit_list(murfey_url, demo=args.demo)
+        ongoing_visits = _get_visit_list(murfey_url)
         pprint(ongoing_visits)
         ongoing_visits = [v.name for v in ongoing_visits]
 
@@ -224,7 +232,8 @@ def run():
     log_queue = Queue()
     input_queue = Queue()
 
-    rich_handler = DirectableRichHandler(log_queue, enable_link_path=False)
+    # rich_handler = DirectableRichHandler(log_queue, enable_link_path=False)
+    rich_handler = DirectableRichHandler(enable_link_path=False)
     rich_handler.setLevel(logging.DEBUG if args.debug else logging.INFO)
     ws = murfey.client.websocket.WSApp(server=args.server)
     logging.getLogger().addHandler(rich_handler)
@@ -235,24 +244,7 @@ def run():
 
     log.info("Starting Websocket connection")
 
-    # start_dc = Prompt.ask("Would you like to register a new data collection?", choices=["y", "n"])
-
-    # if start_dc == "y":
-    #     image_directory = str(args.destination)
-    #     image_suffix = Prompt.ask("Enter the image suffix", choices=[".tiff", ".tif", ".mrc", ".eer"])
-    #     visit = str(args.visit)
-    #     dc_params = {
-    #         "type": "start_dc",
-    #         "image_directory": image_directory,
-    #         "image_suffix": image_suffix,
-    #         "visit": visit,
-    #     }
-    #     ws.send(json.dumps(dc_params))
-
     status_bar = StatusBar()
-    source_watcher = murfey.client.watchdir.DirWatcher(
-        args.source, settling_time=1, status_bar=status_bar
-    )
 
     machine_data = requests.get(f"{murfey_url.geturl()}/machine/").json()
     gain_ref: Path | None = None
@@ -262,18 +254,11 @@ def run():
         except RuntimeError:
             pass
 
-    main_loop_thread = Thread(
-        target=main_loop,
-        args=[source_watcher, args.appearance_time, args.transfer_all],
-        kwargs={"gain_ref": gain_ref},
-        daemon=True,
-    )
-    main_loop_thread.start()
-
     instance_environment = MurfeyInstanceEnvironment(
         url=murfey_url,
-        source=Path(args.source),
-        watcher=source_watcher,
+        software_versions=machine_data.get("software_versions", {}),
+        # sources=[Path(args.source)],
+        # watchers=source_watchers,
         default_destination=args.destination
         or f"{machine_data.get('rsync_module') or 'data'}/{datetime.now().year}",
         demo=args.demo,
@@ -282,70 +267,38 @@ def run():
 
     ws.environment = instance_environment
 
-    rsync_process = RSyncer(
-        instance_environment.source,
-        basepath_remote=Path(args.destination or f"data/{datetime.now().year}"),
-        server_url=murfey_url,
-        local=args.local or instance_environment.demo,
-        do_transfer=not args.no_transfer,
-    )
-    source_watcher.subscribe(rsync_process.enqueue)
-
-    analyser = Analyser(
-        instance_environment.source,
-        environment=instance_environment if args.real_dc else None,
-        force_mdoc_metadata=args.force_mdoc_metadata,
-    )
-    # source_watcher.subscribe(analyser.enqueue)
-    rsync_process.subscribe(analyser.enqueue)
-
     rich_handler.redirect = True
-    MurfeyTUI.run(
-        log_verbosity=2,
+    app = MurfeyTUI(
         environment=instance_environment,
         visits=ongoing_visits,
         queues={"input": input_queue, "logs": log_queue},
         status_bar=status_bar,
-        dummy_dc=not args.real_dc,
+        dummy_dc=args.fake_dc,
         do_transfer=not args.no_transfer,
-        rsync_process=rsync_process,
-        analyser=analyser,
+        gain_ref=gain_ref,
+        redirected_logger=rich_handler,
+        force_mdoc_metadata=not args.ignore_mdoc_metadata,
+        strict=not args.relax,
     )
+    app.run()
     rich_handler.redirect = False
 
-    try:
-        main_loop_thread.join()
-    except KeyboardInterrupt:
-        log.info("Encountered CTRL+C")
-        # if args.destination:
-        #     rsync_process.stop()
-        ws.close()
-        log.info("Client stopped")
-
 
 def main_loop(
-    source_watcher: murfey.client.watchdir.DirWatcher,
+    source_watchers: List[murfey.client.watchdir.DirWatcher],
     appearance_time: float,
     transfer_all: bool,
-    gain_ref: Path | None = None,
 ):
     log.info(
         f"Murfey {murfey.__version__} on Python {'.'.join(map(str, sys.version_info[0:3]))} entering main loop"
     )
-    if gain_ref:
-        gain_rsync = procrunner.run(["rsync", str(gain_ref)])
-        if gain_rsync.returncode:
-            log.warning(
-                f"Gain reference file {gain_ref} was not successfully transferred"
-            )
     if appearance_time > 0:
         modification_time: float | None = time.time() - appearance_time * 3600
     else:
         modification_time = None
     while True:
-        source_watcher.scan(
-            modification_time=modification_time, transfer_all=transfer_all
-        )
+        for sw in source_watchers:
+            sw.scan(modification_time=modification_time, transfer_all=transfer_all)
         time.sleep(15)
 
 
