@@ -4,12 +4,14 @@ import logging
 import queue
 import threading
 from pathlib import Path
+from typing import Type
 
 from murfey.client.context import Context, SPAContext, TomographyContext
 from murfey.client.instance_environment import MurfeyInstanceEnvironment
 from murfey.client.rsync import RSyncerUpdate
 from murfey.client.tui.forms import TUIFormValue
 from murfey.util import Observer
+from murfey.util.models import DCParametersSPA, DCParametersTomo
 
 logger = logging.getLogger("murfey.client.analyser")
 
@@ -32,6 +34,9 @@ class Analyser(Observer):
         self._batch_store: dict = {}
         self._environment = environment
         self._force_mdoc_metadata = force_mdoc_metadata
+        self.parameters_model: Type[DCParametersSPA] | Type[
+            DCParametersTomo
+        ] | None = None
 
         self.queue: queue.Queue = queue.Queue()
         self.thread = threading.Thread(name="Analyser", target=self._analyse)
@@ -45,6 +50,9 @@ class Analyser(Observer):
         ):
             logger.info(f"File extension determined: {file_path.suffix}")
             self._extension = file_path.suffix
+        elif file_path.suffix in (".tiff", ".tif", ".eer"):
+            logger.info(f"File extension re-evaluated: {file_path.suffix}")
+            self._extension = file_path.suffix
 
     def _find_context(self, file_path: Path) -> bool:
         split_file_name = file_path.name.split("_")
@@ -53,47 +61,66 @@ class Analyser(Observer):
                 split_file_name[0] == "Position"
                 or "[" in file_path.name
                 or "Fractions" in split_file_name[-1]
+                or "fractions" in split_file_name[-1]
             ):
                 logger.info("Acquisition software: tomo")
-                self._context = TomographyContext("tomo")
-                if "Fractions" in split_file_name[-1]:
-                    self._role = "detector"
-                elif (
-                    file_path.suffix == ".mdoc"
-                    or file_path.with_suffix(".mdoc").is_file()
-                ):
-                    self._role = "microscope"
-                else:
-                    self._role = "detector"
+                self._context = TomographyContext("tomo", self._basepath)
+                self.parameters_model = DCParametersTomo
+                if not self._role:
+                    if (
+                        "Fractions" in split_file_name[-1]
+                        or "fractions" in split_file_name[-1]
+                    ):
+                        self._role = "detector"
+                    elif (
+                        file_path.suffix == ".mdoc"
+                        or file_path.with_suffix(".mdoc").is_file()
+                    ):
+                        self._role = "microscope"
+                    else:
+                        self._role = "detector"
                 return True
             if split_file_name[0].startswith("FoilHole"):
-                self._context = SPAContext("epu")
-                self._role = "detector"
+                self._context = SPAContext("epu", self._basepath)
+                self.parameters_model = DCParametersSPA
+                if not self._role:
+                    self._role = "detector"
                 return True
             if file_path.suffix in (".mrc", ".tiff", ".tif", ".eer"):
-                self._context = TomographyContext("serialem")
-                if "Frames" in file_path.parts:
-                    self._role = "detector"
-                else:
-                    self._role = "microscope"
+                self._context = TomographyContext("serialem", self._basepath)
+                self.parameters_model = DCParametersTomo
+                if not self._role:
+                    if "Frames" in file_path.parts:
+                        self._role = "detector"
+                    else:
+                        self._role = "microscope"
                 return True
         return False
 
     def _analyse(self):
         logger.info("Analyser thread started")
+        mdoc_for_reading = None
         while not self._halt_thread:
             transferred_file = self.queue.get()
+            logger.info(f"analysing file {transferred_file}")
             if not transferred_file:
                 self._halt_thread = True
                 continue
             dc_metadata = {}
-            if self._force_mdoc_metadata and transferred_file.suffix == ".mdoc":
-                dc_metadata = self._context.gather_metadata(transferred_file)
             if (
-                not self._context
-            ):  # self._experiment_type or not self._acquisition_software:
-                if not self._extension:
-                    self._find_extension(transferred_file)
+                self._force_mdoc_metadata
+                and transferred_file.suffix == ".mdoc"
+                or mdoc_for_reading
+            ):
+                if self._context:
+                    dc_metadata = self._context.gather_metadata(
+                        mdoc_for_reading or transferred_file,
+                        environment=self._environment,
+                    )
+                elif transferred_file.suffix == ".mdoc":
+                    mdoc_for_reading = transferred_file
+            if not self._context:
+                self._find_extension(transferred_file)
                 found = self._find_context(transferred_file)
                 if not found:
                     # logger.warning(
@@ -102,31 +129,35 @@ class Analyser(Observer):
                     continue
                 elif self._extension:
                     logger.info(f"Context found successfully: {self._role}")
-                    self._context.post_first_transfer(
-                        transferred_file, role=self._role, environment=self._environment
-                    )
+                    try:
+                        self._context.post_first_transfer(
+                            transferred_file,
+                            role=self._role,
+                            environment=self._environment,
+                        )
+                    except Exception as e:
+                        logger.info(f"exception encountered {e}")
                     if self._role == "detector":
                         if not dc_metadata:
                             try:
                                 dc_metadata = self._context.gather_metadata(
                                     transferred_file.with_suffix(".mdoc")
                                     if self._context._acquisition_software == "serialem"
-                                    else transferred_file.with_suffix(".xml")
+                                    else transferred_file.with_suffix(".xml"),
+                                    environment=self._environment,
                                 )
                             except NotImplementedError:
                                 dc_metadata = {}
-                        if not dc_metadata:
+                        if not dc_metadata or not self._force_mdoc_metadata:
                             self._unseen_xml.append(transferred_file)
-                            # continue
                         else:
                             self._unseen_xml = []
-                            self.notify({"allowed_responses": ["y", "n"]})
-                            dc_metadata["tilt"] = TUIFormValue(
-                                transferred_file.stem.split("_")[1]
-                            )
-                            dc_metadata["file_extension"] = TUIFormValue(
-                                self._extension
-                            )
+                            if dc_metadata.get("file_extension"):
+                                self._extension = dc_metadata["file_extension"].data
+                            else:
+                                dc_metadata["file_extension"] = TUIFormValue(
+                                    self._extension
+                                )
                             dc_metadata["acquisition_software"] = TUIFormValue(
                                 self._context._acquisition_software
                             )
@@ -137,30 +168,36 @@ class Analyser(Observer):
                     logger.info(
                         f"Context found successfully: {self._role}, {transferred_file}"
                     )
-                    self._context.post_first_transfer(
-                        transferred_file, role=self._role, environment=self._environment
-                    )
+                    try:
+                        self._context.post_first_transfer(
+                            transferred_file,
+                            role=self._role,
+                            environment=self._environment,
+                        )
+                    except Exception as e:
+                        logger.info(f"exception encountered {e}")
                     if self._role == "detector":
                         if not dc_metadata:
                             dc_metadata = self._context.gather_metadata(
-                                transferred_file.with_suffix(".xml")
+                                mdoc_for_reading
+                                or transferred_file.with_suffix(".xml"),
+                                environment=self._environment,
                             )
-                        if not dc_metadata:
+                        if not dc_metadata or not self._force_mdoc_metadata:
                             self._unseen_xml.append(transferred_file)
-                        else:
+                        if dc_metadata:
                             self._unseen_xml = []
-                            self.notify({"allowed_responses": ["y", "n"]})
-                            dc_metadata["tilt"] = TUIFormValue(
-                                transferred_file.stem.split("_")[1]
-                            )
-                            dc_metadata["file_extension"] = TUIFormValue(
-                                self._extension
-                            )
+                            if dc_metadata.get("file_extension"):
+                                self._extension = dc_metadata["file_extension"].data
+                            else:
+                                dc_metadata["file_extension"] = TUIFormValue(
+                                    self._extension
+                                )
                             dc_metadata["acquisition_software"] = TUIFormValue(
                                 self._context._acquisition_software
                             )
                             self.notify({"form": dc_metadata})
-            else:
+            elif isinstance(self._context, TomographyContext):
                 _tilt_series = set(self._context._tilt_series.keys())
                 self._context.post_transfer(
                     transferred_file, role=self._role, environment=self._environment
@@ -170,8 +207,10 @@ class Analyser(Observer):
                     and self._role == "detector"
                 ):
                     if not dc_metadata:
+                        logger.info("gather metadata 4")
                         dc_metadata = self._context.gather_metadata(
-                            transferred_file.with_suffix(".xml")
+                            transferred_file.with_suffix(".xml"),
+                            environment=self._environment,
                         )
                     self.notify({"form": dc_metadata})
 
@@ -179,7 +218,6 @@ class Analyser(Observer):
         if not self._stopping:
             absolute_path = (self._basepath / rsyncer.file_path).resolve()
             self.queue.put(absolute_path)
-            # self.queue.put(file_path)
 
     def start(self):
         if self.thread.is_alive():
