@@ -1,14 +1,16 @@
 from __future__ import annotations
 
 import logging
+from functools import lru_cache
 from pathlib import Path
 from threading import RLock
-from typing import Callable, Dict, List, OrderedDict
+from typing import Any, Callable, Dict, List, NamedTuple, Optional, OrderedDict
 
 import requests
 import xmltodict
 from pydantic import BaseModel
 
+from murfey.client.contexts.tomo import tomo_tilt_info
 from murfey.client.instance_environment import (
     MovieID,
     MovieTracker,
@@ -17,13 +19,39 @@ from murfey.client.instance_environment import (
     global_env_lock,
 )
 from murfey.client.tui.forms import TUIFormValue
-from murfey.util.mdoc import get_global_data
-
-# import time
+from murfey.util.mdoc import get_block, get_global_data, get_num_blocks
 
 logger = logging.getLogger("murfey.client.context")
 
 
+class FutureRequest(NamedTuple):
+    url: str
+    message: Dict[str, Any]
+
+
+class ProcessingParameter(NamedTuple):
+    name: str
+    label: str
+    default: Any = None
+
+
+@lru_cache(maxsize=5)
+def get_machine_config(url: str, demo: bool = False):
+    if demo:
+        return {}
+    return requests.get(f"{url}/machine/").json()
+
+
+def _construct_tilt_series_name(
+    tilt_tag: str, tilt_series: str, file_path: Path
+) -> str:
+    if tilt_tag:
+        if f"{tilt_tag}_{tilt_series}" in file_path.name:
+            return f"{tilt_tag}_{tilt_series}"
+        return f"{tilt_tag}{tilt_series}"
+    return tilt_series
+
+
 def detect_acquisition_software(dir_for_transfer: Path) -> str:
     glob = dir_for_transfer.glob("*")
     for f in glob:
@@ -35,6 +63,9 @@ def detect_acquisition_software(dir_for_transfer: Path) -> str:
 
 
 class Context:
+    user_params: List[ProcessingParameter] = []
+    metadata_params: List[ProcessingParameter] = []
+
     def __init__(self, acquisition_software: str):
         self._acquisition_software = acquisition_software
 
@@ -46,15 +77,175 @@ class Context:
     def post_first_transfer(self, transferred_file: Path, role: str = "", **kwargs):
         self.post_transfer(transferred_file, role=role, **kwargs)
 
-    def gather_metadata(self, metadata_file: Path):
+    def gather_metadata(
+        self, metadata_file: Path, environment: MurfeyInstanceEnvironment | None = None
+    ):
         raise NotImplementedError(
             f"gather_metadata must be declared in derived class to be used: {self}"
         )
 
 
 class SPAContext(Context):
-    def post_transfer(self, transferred_file: Path, role: str = "", **kwargs):
-        pass
+    user_params = [
+        ProcessingParameter(
+            "dose_per_frame", "Dose Per Frame (e- / Angstrom^2 / frame)"
+        ),
+        ProcessingParameter("use_cryolo", "Use crYOLO Autopicking", default=True),
+        ProcessingParameter("symmetry", "Symmetry Group", default="C1"),
+        ProcessingParameter("boxsize", "Box Size", default=256),
+        ProcessingParameter("eer_grouping", "EER Grouping", default=20),
+        ProcessingParameter(
+            "mask_diameter", "Mask Diameter (2D classification)", default=190
+        ),
+        ProcessingParameter("downscale", "Downscale Extracted Particles", default=True),
+        ProcessingParameter(
+            "small_boxsize", "Downscaled Extracted Particle Size (pixels)", default=128
+        ),
+    ]
+    metadata_params = [
+        ProcessingParameter("voltage", "Voltage"),
+        ProcessingParameter("image_size_x", "Image Size X"),
+        ProcessingParameter("image_size_y", "Image Size Y"),
+        ProcessingParameter("pixel_size_on_image", "Pixel Size"),
+        ProcessingParameter("motion_corr_binning", "Motion Correction Binning"),
+        ProcessingParameter("gain_ref", "Gain Reference"),
+    ]
+
+    def __init__(self, acquisition_software: str, basepath: Path):
+        super().__init__(acquisition_software)
+        self._basepath = basepath
+        self._processing_job_stash: dict = {}
+        self._preprocessing_triggers: dict = {}
+
+    def _flush_processing_job(self, tag: str, parameters: Dict[str, Any] | None = None):
+        if parameters:
+            proc_job_future = self._processing_job_stash[tag]
+            msg = {
+                **proc_job_future.message,
+                "parameters": {
+                    "acquisition_software": parameters["acquisition_software"],
+                    "voltage": parameters["voltage"],
+                    "motioncor_gainreference": parameters["gain_ref"],
+                    "motioncor_doseperframe": parameters["dose_per_frame"],
+                    "eer_grouping": parameters["eer_grouping"],
+                    "import_images": f"{Path(parameters['image_directory']).resolve()}/*{parameters['file_extension']}",
+                    "angpix": parameters["pixel_size_on_image"],
+                    "symmetry": parameters["symmetry"],
+                    "extract_boxsize": parameters["boxsize"],
+                    "extract_downscale": parameters["downscale"],
+                    "extract_small_boxsize": parameters["small_boxsize"],
+                    "mask_diameter": parameters["mask_diameter"],
+                    "autopick_do_cryolo": parameters["use_cryolo"],
+                },
+            }
+            requests.post(proc_job_future.url, json=msg)
+
+    def _register_data_collection(self, url: str, data: dict):
+        logger.info(f"registering data collection with data {data}")
+        json = {
+            "voltage": data["voltage"],
+            "pixel_size_on_image": data["pixel_size_on_image"],
+            "experiment_type": data["experiment_type"],
+            "image_size_x": data["image_size_x"],
+            "image_size_y": data["image_size_y"],
+            "file_extension": data["file_extension"],
+            "acquisition_software": data["acquisition_software"],
+            "image_directory": data["image_directory"],
+            "tag": data["tag"],
+        }
+        requests.post(url, json=json)
+
+    def post_transfer(
+        self,
+        transferred_file: Path,
+        role: str = "",
+        environment: MurfeyInstanceEnvironment | None = None,
+        **kwargs,
+    ):
+        if role == "detector" and environment:
+            source = ""
+            for s in environment.sources:
+                if transferred_file.is_relative_to(s):
+                    source = str(s)
+                    break
+            proc_url = f"{str(environment.url.geturl())}/visits/{environment.visit}/register_processing_job"
+            if environment.data_collection_ids.get(source) is None:
+                self._processing_job_stash[source] = FutureRequest(
+                    proc_url,
+                    {"tag": source, "recipe": "relion"},
+                )
+            elif environment.data_collection_parameters:
+                if self._processing_job_stash.get(source):
+                    # self._flush_processing_job(source)
+                    pass
+                requests.post(
+                    proc_url,
+                    json={
+                        "tag": source,
+                        "recipe": "relion",
+                        "parameters": environment.data_collection_parameters,
+                    },
+                )
+
+    def _register_processing_job(
+        self, tag: str, parameters: Dict[str, Any] | None = None
+    ):
+        if self._processing_job_stash.get(tag):
+            self._flush_processing_job(tag, parameters=parameters)
+            self._processing_job_stash.pop(tag)
+
+    def _launch_spa_pipeline(self, tag: str, jobid: int, url: str = ""):
+        data = {"job_id": jobid}
+        requests.post(url, json=data)
+
+    def gather_metadata(
+        self, metadata_file: Path, environment: MurfeyInstanceEnvironment | None = None
+    ):
+        if metadata_file.suffix != ".xml":
+            raise ValueError(
+                f"SPA gather_metadata method expected xml file not {metadata_file.name}"
+            )
+        if not metadata_file.is_file():
+            logger.debug(f"Metadata file {metadata_file} not found")
+            return OrderedDict({})
+        with open(metadata_file, "r") as xml:
+            try:
+                for_parsing = xml.read()
+            except Exception:
+                logger.warning(f"Failed to parse file {metadata_file}")
+                return OrderedDict({})
+            data = xmltodict.parse(for_parsing)
+        metadata: OrderedDict = OrderedDict({})
+        metadata["experiment_type"] = TUIFormValue("SPA")
+        metadata["voltage"] = TUIFormValue(300)
+        metadata["image_size_x"] = TUIFormValue(
+            data["Acquisition"]["Info"]["ImageSize"]["Width"]
+        )
+        metadata["image_size_y"] = TUIFormValue(
+            data["Acquisition"]["Info"]["ImageSize"]["Height"]
+        )
+        metadata["pixel_size_on_image"] = TUIFormValue(
+            float(data["Acquisition"]["Info"]["SensorPixelSize"]["Height"])
+        )
+        metadata["motion_corr_binning"] = TUIFormValue(1)
+        metadata["gain_ref"] = TUIFormValue(None, top=True)
+        metadata["dose_per_frame"] = TUIFormValue(
+            environment.data_collection_parameters.get("dose_per_frame")
+            if environment
+            else None,
+            top=True,
+        )
+        metadata["use_cryolo"] = TUIFormValue(True)
+        metadata["symmetry"] = TUIFormValue("C1")
+        metadata["mask_diameter"] = TUIFormValue(190)
+        metadata["boxsize"] = TUIFormValue(256)
+        metadata["downscale"] = TUIFormValue(False)
+        metadata["small_boxsize"] = TUIFormValue(128)
+        metadata["eer_grouping"] = TUIFormValue(20)
+        metadata["source"] = TUIFormValue(str(self._basepath))
+        metadata.move_to_end("gain_ref", last=False)
+        metadata.move_to_end("dose_per_frame", last=False)
+        return metadata
 
 
 class ProcessFileIncomplete(BaseModel):
@@ -67,9 +258,26 @@ class ProcessFileIncomplete(BaseModel):
 
 
 class TomographyContext(Context):
-    def __init__(self, acquisition_software: str):
+    user_params = [
+        ProcessingParameter(
+            "dose_per_frame", "Dose Per Frame (e- / Angstrom^2 / frame)"
+        ),
+        ProcessingParameter("tilt_offset", "Tilt Offset"),
+    ]
+    metadata_params = [
+        ProcessingParameter("voltage", "Voltage"),
+        ProcessingParameter("image_size_x", "Image Size X"),
+        ProcessingParameter("image_size_y", "Image Size Y"),
+        ProcessingParameter("pixel_size_on_image", "Pixel Size"),
+        ProcessingParameter("motion_corr_binning", "Motion Correction Binning"),
+        ProcessingParameter("gain_ref", "Gain Reference"),
+    ]
+
+    def __init__(self, acquisition_software: str, basepath: Path):
         super().__init__(acquisition_software)
+        self._basepath = basepath
         self._tilt_series: Dict[str, List[Path]] = {}
+        self._tilt_series_sizes: Dict[str, int | None] = {}
         self._completed_tilt_series: List[str] = []
         self._aligned_tilt_series: List[str] = []
         self._motion_corrected_tilt_series: Dict[str, List[Path]] = {}
@@ -89,16 +297,12 @@ class TomographyContext(Context):
         self._data_collection_stash = []
 
     def _flush_processing_job(self, tag: str):
-        # logger.info(
-        #     f"Flushing processing job {tag}, {self._processing_job_stash.get(tag)}"
-        # )
         if proc_data := self._processing_job_stash.get(tag):
             for pd in proc_data:
                 requests.post(pd[0], json=pd[1])
             self._processing_job_stash.pop(tag)
 
     def _flush_preprocess(self, tag: str, app_id: int):
-        # logger.info(f"Flushing preprocessing requests {tag}")
         if tag_tr := self._preprocessing_triggers.get(tag):
             for tr in tag_tr:
                 process_file = self._complete_process_file(tr[1], tr[2], app_id)
@@ -116,11 +320,15 @@ class TomographyContext(Context):
         appid: int,
         mvid: int,
         tilt_angles: List,
+        manual_tilt_offset: Optional[float],
+        pixel_size: Optional[float],
     ):
         if self._extract_tilt_series and self._extract_tilt_tag:
-            tilt_series = self._extract_tilt_tag(
-                movie_path
-            ) + self._extract_tilt_series(movie_path)
+            tilt_series = (
+                f"{self._extract_tilt_tag(movie_path)}_{self._extract_tilt_series(movie_path)}"
+                if self._extract_tilt_tag(movie_path)
+                else self._extract_tilt_series(movie_path)
+            )
         else:
             return
 
@@ -151,6 +359,8 @@ class TomographyContext(Context):
                         "autoproc_program_id": appid,
                         "motion_corrected_path": str(motion_corrected_path),
                         "movie_id": mvid,
+                        "manual_tilt_offset": manual_tilt_offset,
+                        "pixel_size": pixel_size,
                     }
                     requests.post(url, json=series_data)
                     with self._lock:
@@ -183,6 +393,13 @@ class TomographyContext(Context):
                     ],
                     "autoproc_program_id": app_id,
                     "mc_uuid": incomplete_process_file.mc_uuid,
+                    "dose_per_frame": environment.data_collection_parameters.get(
+                        "dose_per_frame"
+                    ),
+                    "mc_binning": environment.data_collection_parameters.get(
+                        "motion_corr_binning", 1
+                    ),
+                    "gain_ref": environment.data_collection_parameters.get("gain_ref"),
                 }
                 return new_dict
         except KeyError:
@@ -196,12 +413,30 @@ class TomographyContext(Context):
         extract_tilt_angle: Callable[[Path], str],
         extract_tilt_tag: Callable[[Path], str],
         environment: MurfeyInstanceEnvironment | None = None,
+        required_position_files: List[Path] | None = None,
+        required_strings: List[str] | None = None,
     ) -> List[str]:
+        if not environment:
+            logger.warning("No environment passed in")
+            return []
+        for s in environment.sources:
+            if file_path.is_relative_to(s):
+                source = s
+                break
+        else:
+            logger.warning(f"No source found for file {file_path}")
+            return []
+        # required_position_files = required_position_files or []
+        required_strings = (
+            ["fractions"] if required_strings is None else required_strings
+        )
+        for r in required_strings:
+            if r not in file_path.name.lower():
+                return []
         if not self._extract_tilt_series:
             self._extract_tilt_series = extract_tilt_series
         if not self._extract_tilt_tag:
             self._extract_tilt_tag = extract_tilt_tag
-        # time.sleep(5)
         try:
             tilt_series_num = extract_tilt_series(file_path)
             tilt_angle = extract_tilt_angle(file_path)
@@ -211,30 +446,30 @@ class TomographyContext(Context):
                 float(tilt_angle)
             except ValueError:
                 return []
-            tilt_series = tilt_tag + tilt_series_num
+            tilt_series = _construct_tilt_series_name(
+                tilt_tag, tilt_series_num, file_path
+            )
 
         except Exception:
-            logger.info(
+            logger.debug(
                 f"Tilt series and angle could not be determined for {file_path}"
             )
             return []
 
         if environment:
-            machine_config = (
-                {}
-                if environment.demo
-                else requests.get(f"{str(environment.url.geturl())}/machine/").json()
+            machine_config = get_machine_config(
+                str(environment.url.geturl()), demo=environment.demo
             )
-            if environment.visit in environment.default_destination:
+            if environment.visit in environment.default_destinations[source]:
                 file_transferred_to = (
                     Path(machine_config.get("rsync_basepath", ""))
-                    / Path(environment.default_destination)
+                    / Path(environment.default_destinations[source])
                     / file_path.name
                 )
             else:
                 file_transferred_to = (
                     Path(machine_config.get("rsync_basepath", ""))
-                    / Path(environment.default_destination)
+                    / Path(environment.default_destinations[source])
                     / environment.visit
                     / file_path.name
                 )
@@ -263,12 +498,13 @@ class TomographyContext(Context):
         if not self._tilt_series.get(tilt_series):
             logger.info(f"New tilt series found: {tilt_series}")
             self._tilt_series[tilt_series] = [file_path]
+            if not self._tilt_series_sizes.get(tilt_series):
+                self._tilt_series_sizes[tilt_series] = 0
             try:
                 if environment:
                     url = f"{str(environment.url.geturl())}/visits/{environment.visit}/start_data_collection"
                     data = {
                         "experiment_type": "tomography",
-                        "tilt": tilt_series_num,
                         "file_extension": file_path.suffix,
                         "acquisition_software": self._acquisition_software,
                         "image_directory": str(file_path.parent),
@@ -291,7 +527,7 @@ class TomographyContext(Context):
                                 ],
                             }
                         )
-                    if environment.data_collection_group_id is None:
+                    if environment.data_collection_group_ids.get(tilt_series) is None:
                         self._data_collection_stash.append((url, environment, data))
                     else:
                         requests.post(url, json=data)
@@ -321,7 +557,11 @@ class TomographyContext(Context):
                 logger.error(f"ERROR {e}")
         else:
             if file_path not in self._tilt_series[tilt_series]:
-                self._tilt_series[tilt_series].append(file_path)
+                for p in self._tilt_series[tilt_series]:
+                    if tilt_angle == extract_tilt_angle(p):
+                        break
+                else:
+                    self._tilt_series[tilt_series].append(file_path)
 
         if environment and environment.autoproc_program_ids.get(tilt_series):
             preproc_url = f"{str(environment.url.geturl())}/visits/{environment.visit}/tomography_preprocess"
@@ -344,13 +584,20 @@ class TomographyContext(Context):
                 "mc_uuid": environment.movies[
                     file_transferred_to
                 ].motion_correction_uuid,
+                "dose_per_frame": environment.data_collection_parameters.get(
+                    "dose_per_frame"
+                ),
+                "mc_binning": environment.data_collection_parameters.get(
+                    "motion_corr_binning", 1
+                ),
+                "gain_ref": environment.data_collection_parameters.get("gain_ref"),
             }
             requests.post(preproc_url, json=preproc_data)
         elif environment:
             preproc_url = f"{str(environment.url.geturl())}/visits/{environment.visit}/tomography_preprocess"
             pfi = ProcessFileIncomplete(
                 dest=file_transferred_to,
-                source=environment.source,
+                source=source,
                 image_number=environment.movies[file_transferred_to].movie_number,
                 mc_uuid=environment.movies[file_transferred_to].motion_correction_uuid,
                 tag=tilt_series,
@@ -380,26 +627,42 @@ class TomographyContext(Context):
                     ]
 
         if self._last_transferred_file:
-            last_tilt_series = extract_tilt_tag(
-                self._last_transferred_file
-            ) + extract_tilt_series(self._last_transferred_file)
+            last_tilt_series = (
+                f"{extract_tilt_tag(self._last_transferred_file)}_{extract_tilt_series(self._last_transferred_file)}"
+                if extract_tilt_tag(self._last_transferred_file)
+                else extract_tilt_series(self._last_transferred_file)
+            )
             last_tilt_angle = extract_tilt_angle(self._last_transferred_file)
             self._last_transferred_file = file_path
-            if last_tilt_series != tilt_series and last_tilt_angle != tilt_angle:
+            if (
+                last_tilt_series != tilt_series
+                and last_tilt_angle != tilt_angle
+                or self._tilt_series_sizes.get(tilt_series)
+            ) or self._completed_tilt_series:
                 newly_completed_series = []
                 if self._tilt_series:
                     tilt_series_size = max(len(ts) for ts in self._tilt_series.values())
                 else:
                     tilt_series_size = 0
                 this_tilt_series_size = len(self._tilt_series[tilt_series])
-                if this_tilt_series_size >= tilt_series_size:
+                tilt_series_size_check = (
+                    (this_tilt_series_size == self._tilt_series_sizes.get(tilt_series))
+                    if self._tilt_series_sizes.get(tilt_series)
+                    else (this_tilt_series_size >= tilt_series_size)
+                )
+                if tilt_series_size_check and not required_position_files:
                     self._completed_tilt_series.append(tilt_series)
                     newly_completed_series.append(tilt_series)
                 for ts, ta in self._tilt_series.items():
-                    if (
-                        len(ta) >= tilt_series_size
-                        and ts not in self._completed_tilt_series
-                    ):
+                    if self._tilt_series_sizes.get(ts):
+                        completion_test = len(ta) == self._tilt_series_sizes[ts]
+                    elif required_position_files:
+                        completion_test = all(
+                            _f.is_file() for _f in required_position_files
+                        )
+                    else:
+                        completion_test = len(ta) >= tilt_series_size
+                    if ts not in self._completed_tilt_series and completion_test:
                         newly_completed_series.append(ts)
                         self._completed_tilt_series.append(ts)
                         if environment:
@@ -449,6 +712,12 @@ class TomographyContext(Context):
                                             ][1]
                                         ),
                                         file_tilt_list,
+                                        environment.data_collection_parameters.get(
+                                            "manual_tilt_offset"
+                                        ),
+                                        environment.data_collection_parameters.get(
+                                            "pixel_size_on_image"
+                                        ),
                                     )
                 if newly_completed_series:
                     logger.info(
@@ -459,49 +728,42 @@ class TomographyContext(Context):
         return []
 
     def _add_tomo_tilt(
-        self, file_path: Path, environment: MurfeyInstanceEnvironment | None = None
+        self,
+        file_path: Path,
+        environment: MurfeyInstanceEnvironment | None = None,
+        required_position_files: List[Path] | None = None,
+        required_strings: List[str] | None = None,
     ) -> List[str]:
-        if "[" in file_path.name:
-            return self._add_tilt(
-                file_path,
-                lambda x: x.name.split("_")[1],
-                lambda x: x.name.split("[")[1].split("]")[0],
-                lambda x: x.name.split("_")[0],
-                environment=environment,
-            )
-
-        def _get_slice_index(tag: str) -> int:
-            slice_index = 0
-            for i, ch in enumerate(tag[::-1]):
-                if not ch.isnumeric():
-                    slice_index = -i
-                    break
-            if not slice_index:
-                raise ValueError(
-                    f"The file tag {tag} does not end in numeric characters or is entirely numeric: cannot parse"
-                )
-            return slice_index
-
-        def _extract_tilt_series(p: Path) -> str:
-            tag = p.name.split("_")[0]
-            slice_index = _get_slice_index(tag)
-            return tag[slice_index:]
-
-        def _extract_tilt_tag(p: Path) -> str:
-            tag = p.name.split("_")[0]
-            slice_index = _get_slice_index(tag)
-            return tag[:slice_index]
-
-        def _extract_tilt_angle(p: Path) -> str:
-            _split = p.name.split("_")[2].split(".")
-            return ".".join(_split[:-1])
-
+        required_strings = (
+            ["fractions"] if required_strings is None else required_strings
+        )
+        for r in required_strings:
+            if r not in file_path.name.lower():
+                return []
+        if environment:
+            if tomo_version := environment.software_versions.get("tomo"):
+                tilt_info_extraction = tomo_tilt_info.get(tomo_version)
+                if not tilt_info_extraction:
+                    raise ValueError(
+                        f"Extraction routines for TFS Tomo version {tomo_version} unknown"
+                    )
+            else:
+                tilt_info_extraction = tomo_tilt_info["5.7"]
+        else:
+            tilt_info_extraction = tomo_tilt_info["5.7"]
+        tilt_tag = tilt_info_extraction.tag(file_path)
+        tilt_series_num = tilt_info_extraction.series(file_path)
+        tilt_series = _construct_tilt_series_name(tilt_tag, tilt_series_num, file_path)
         return self._add_tilt(
             file_path,
-            _extract_tilt_series,
-            _extract_tilt_angle,
-            _extract_tilt_tag,
+            tilt_info_extraction.series,
+            tilt_info_extraction.angle,
+            tilt_info_extraction.tag,
             environment=environment,
+            required_position_files=required_position_files
+            if required_position_files is not None
+            else [file_path.parent / (tilt_series + ".mdoc")],
+            required_strings=required_strings,
         )
 
     def _add_serialem_tilt(
@@ -530,6 +792,7 @@ class TomographyContext(Context):
             lambda x: ".".join(x.name.split(delimiter)[-1].split(".")[:-1]),
             lambda x: "",
             environment=environment,
+            required_strings=[],
         )
 
     def post_transfer(
@@ -541,15 +804,33 @@ class TomographyContext(Context):
     ) -> List[str]:
         data_suffixes = (".mrc", ".tiff", ".tif", ".eer")
         completed_tilts = []
-        if role == "detector" and transferred_file.suffix in data_suffixes:
-            if self._acquisition_software == "tomo":
-                completed_tilts = self._add_tomo_tilt(
-                    transferred_file, environment=environment
-                )
-            elif self._acquisition_software == "serialem":
-                completed_tilts = self._add_serialem_tilt(
-                    transferred_file, environment=environment
-                )
+        if role == "detector" and "gain" not in transferred_file.name:
+            if transferred_file.suffix in data_suffixes:
+                if self._acquisition_software == "tomo":
+                    if environment:
+                        machine_config = get_machine_config(
+                            str(environment.url.geturl()), demo=environment.demo
+                        )
+                    else:
+                        machine_config = {}
+                    required_strings = machine_config.get(
+                        "data_required_substrings", {}
+                    ).get("tomo")
+                    completed_tilts = self._add_tomo_tilt(
+                        transferred_file,
+                        environment=environment,
+                        required_position_files=kwargs.get("required_position_files"),
+                        required_strings=required_strings
+                        or kwargs.get("required_strings"),
+                    )
+                elif self._acquisition_software == "serialem":
+                    completed_tilts = self._add_serialem_tilt(
+                        transferred_file, environment=environment
+                    )
+            if transferred_file.suffix == ".mdoc":
+                with open(transferred_file, "r") as md:
+                    tilt_series = transferred_file.stem
+                    self._tilt_series_sizes[tilt_series] = get_num_blocks(md)
         return completed_tilts
 
     def post_first_transfer(
@@ -559,9 +840,13 @@ class TomographyContext(Context):
         environment: MurfeyInstanceEnvironment | None = None,
         **kwargs,
     ):
-        self.post_transfer(transferred_file, role=role, environment=environment)
+        self.post_transfer(
+            transferred_file, role=role, environment=environment, **kwargs
+        )
 
-    def gather_metadata(self, metadata_file: Path) -> OrderedDict:
+    def gather_metadata(
+        self, metadata_file: Path, environment: MurfeyInstanceEnvironment | None = None
+    ) -> OrderedDict:
         if metadata_file.suffix not in (".mdoc", ".xml"):
             raise ValueError(
                 f"Tomography gather_metadata method expected xml or mdoc file not {metadata_file.name}"
@@ -577,26 +862,38 @@ class TomographyContext(Context):
                     logger.warning(f"Failed to parse file {metadata_file}")
                     return OrderedDict({})
                 data = xmltodict.parse(for_parsing)
-            metadata: OrderedDict = OrderedDict({})
-            metadata["experiment_type"] = TUIFormValue("tomography")
-            metadata["voltage"] = TUIFormValue(300)
-            metadata["image_size_x"] = TUIFormValue(
-                data["Acquisition"]["Info"]["ImageSize"]["Width"]
-            )
-            metadata["image_size_y"] = TUIFormValue(
-                data["Acquisition"]["Info"]["ImageSize"]["Height"]
-            )
-            metadata["pixel_size_on_image"] = TUIFormValue(
-                float(data["Acquisition"]["Info"]["SensorPixelSize"]["Height"])
-            )
-            metadata["dose_per_frame"] = TUIFormValue(
-                None, top=True, colour="dark_orange"
-            )
-            metadata.move_to_end("dose_per_frame", last=False)
-            # logger.info(f"Metadata extracted from {metadata_file}: {metadata}")
+            try:
+                metadata: OrderedDict = OrderedDict({})
+                metadata["experiment_type"] = TUIFormValue("tomography")
+                metadata["voltage"] = TUIFormValue(300)
+                metadata["image_size_x"] = TUIFormValue(
+                    data["Acquisition"]["Info"]["ImageSize"]["Width"]
+                )
+                metadata["image_size_y"] = TUIFormValue(
+                    data["Acquisition"]["Info"]["ImageSize"]["Height"]
+                )
+                metadata["pixel_size_on_image"] = TUIFormValue(
+                    float(data["Acquisition"]["Info"]["SensorPixelSize"]["Height"])
+                )
+                metadata["motion_corr_binning"] = TUIFormValue(1)
+                metadata["gain_ref"] = TUIFormValue(None, top=True)
+                metadata["dose_per_frame"] = TUIFormValue(
+                    environment.data_collection_parameters.get("dose_per_frame")
+                    if environment
+                    else None,
+                    top=True,
+                    colour="dark_orange",
+                )
+                metadata["manual_tilt_offset"] = TUIFormValue(0, top=True)
+                metadata["source"] = TUIFormValue(str(self._basepath))
+                metadata.move_to_end("gain_ref", last=False)
+                metadata.move_to_end("dose_per_frame", last=False)
+            except KeyError:
+                return OrderedDict({})
             return metadata
         with open(metadata_file, "r") as md:
             mdoc_data = get_global_data(md)
+            mdoc_data_block = get_block(md)
         if not mdoc_data:
             return OrderedDict({})
         mdoc_metadata: OrderedDict = OrderedDict({})
@@ -604,12 +901,43 @@ class TomographyContext(Context):
         mdoc_metadata["voltage"] = TUIFormValue(float(mdoc_data["Voltage"]))
         mdoc_metadata["image_size_x"] = TUIFormValue(int(mdoc_data["ImageSize"][0]))
         mdoc_metadata["image_size_y"] = TUIFormValue(int(mdoc_data["ImageSize"][1]))
-        mdoc_metadata["pixel_size_on_image"] = TUIFormValue(
-            float(mdoc_data["PixelSpacing"]) * 1e-10
+        mdoc_metadata["magnification"] = TUIFormValue(
+            int(mdoc_data_block["Magnification"])
         )
+        superres_binning = int(mdoc_data_block["Binning"])
+        binning_factor = 1
+        if environment:
+            server_config = requests.get(
+                f"{str(environment.url.geturl())}/machine/"
+            ).json()
+            if server_config.get("superres") and superres_binning == 1:
+                binning_factor = 2
+            ps_from_mag = (
+                server_config.get("calibrations", {})
+                .get("magnification", {})
+                .get(int(mdoc_data_block["Magnification"]))
+            )
+            if ps_from_mag:
+                mdoc_metadata["pixel_size_on_image"] = TUIFormValue(
+                    float(ps_from_mag) * 1e-10 / binning_factor
+                )
+        if mdoc_metadata.get("pixel_size_on_image") is None:
+            mdoc_metadata["pixel_size_on_image"] = TUIFormValue(
+                float(mdoc_data["PixelSpacing"]) * 1e-10 / binning_factor
+            )
+        mdoc_metadata["motion_corr_binning"] = TUIFormValue(binning_factor)
+        mdoc_metadata["gain_ref"] = TUIFormValue(None, top=True)
         mdoc_metadata["dose_per_frame"] = TUIFormValue(
-            None, top=True, colour="dark_orange"
+            environment.data_collection_parameters.get("dose_per_frame")
+            if environment
+            else None,
+            top=True,
         )
+        mdoc_metadata["manual_tilt_offset"] = TUIFormValue(0, top=True)
+        mdoc_metadata["source"] = TUIFormValue(str(self._basepath))
+        mdoc_metadata.move_to_end("gain_ref", last=False)
         mdoc_metadata.move_to_end("dose_per_frame", last=False)
-        # logger.info(f"Metadata extracted from {metadata_file}")
+        mdoc_metadata["file_extension"] = TUIFormValue(
+            f".{mdoc_data_block['SubFramePath'].split('.')[-1]}"
+        )
         return mdoc_metadata
