@@ -27,11 +27,12 @@ global_env_lock = RLock()
 
 class MurfeyInstanceEnvironment(BaseModel):
     url: ParseResult
-    source: Optional[Path] = None
-    default_destination: str = ""
-    watcher: Optional[DirWatcher] = None
+    software_versions: Dict[str, str] = {}
+    sources: List[Path] = []
+    default_destinations: Dict[Path, str] = {}
+    watchers: Dict[Path, DirWatcher] = {}
     demo: bool = False
-    data_collection_group_id: Optional[int] = None
+    data_collection_group_ids: Dict[str, int] = {}
     data_collection_ids: Dict[str, int] = {}
     processing_job_ids: Dict[str, Dict[str, int]] = {}
     autoproc_program_ids: Dict[str, Dict[str, int]] = {}
@@ -43,15 +44,35 @@ class MurfeyInstanceEnvironment(BaseModel):
     tilt_angles: Dict[str, List[List[str]]] = {}
     visit: str = ""
     processing_only_mode: bool = False
+    gain_ref: Optional[Path] = None
 
     class Config:
         validate_assignment: bool = True
         arbitrary_types_allowed: bool = True
 
-    @validator("data_collection_group_id")
+    def clear(self):
+        self.sources = []
+        self.default_destinations = {}
+        for w in self.watchers.values():
+            w.stop()
+        self.watchers = {}
+        self.data_collection_group_ids = {}
+        self.data_collection_ids = {}
+        self.processing_job_ids = {}
+        self.autoproc_program_ids = {}
+        self.data_collection_parameters = {}
+        self.movies = {}
+        self.motion_corrected_movies = {}
+        self.listeners = {}
+        self.movie_tilt_pair = {}
+        self.tilt_angles = {}
+        self.visit = ""
+        self.gain_ref = None
+
+    @validator("data_collection_group_ids")
     def dcg_callback(cls, v, values):
         with global_env_lock:
-            for l in values.get("listeners", {}).get("data_collection_group_id", []):
+            for l in values.get("listeners", {}).get("data_collection_group_ids", []):
                 l()
         return v
 
@@ -67,6 +88,18 @@ class MurfeyInstanceEnvironment(BaseModel):
                         l(k)
         return v
 
+    @validator("processing_job_ids")
+    def job_callback(cls, v, values):
+        with global_env_lock:
+            for l in values.get("listeners", {}).get("processing_job_ids", []):
+                if values.get("processing_job_ids"):
+                    for k in set(values["processing_job_ids"].keys()) ^ set(v.keys()):
+                        l(k, v[k]["relion"])
+                else:
+                    for k in v.keys():
+                        l(k, v[k]["relion"])
+        return v
+
     @validator("autoproc_program_ids")
     def app_callback(cls, v, values):
         # logger.info(f"autoproc program ids validator: {v}")
@@ -113,6 +146,8 @@ class MurfeyInstanceEnvironment(BaseModel):
                         values["autoproc_program_ids"][k]["em-tomo-align"],
                         v[k][1],
                         file_tilt_list,
+                        values["data_collection_parameters"]["manual_tilt_offset"],
+                        values["data_collection_parameters"]["pixel_size_on_image"],
                     )
             else:
                 for k in v.keys():
@@ -139,6 +174,8 @@ class MurfeyInstanceEnvironment(BaseModel):
                             values["autoproc_program_ids"][tilt]["em-tomo-align"],
                             v[k][1],
                             file_tilt_list,
+                            values["data_collection_parameters"]["manual_tilt_offset"],
+                            values["data_collection_parameters"]["pixel_size_on_image"],
                         )
                     except KeyError:
                         pass
