@@ -52,10 +52,12 @@ class RSyncer(Observer):
         local: bool = False,
         status_bar: StatusBar | None = None,
         do_transfer: bool = True,
+        remove_files: bool = False,
     ):
         super().__init__()
         self._basepath = basepath_local.absolute()
         self._do_transfer = do_transfer
+        self._remove_files = remove_files
         if local:
             self._remote = str(basepath_remote)
         else:
@@ -140,7 +142,9 @@ class RSyncer(Observer):
             except queue.Empty:
                 pass
 
-            logger.info(f"Preparing to transfer {len(files_to_transfer)} files")
+            logger.info(
+                f"Preparing to transfer {len(files_to_transfer)} files: {files_to_transfer}, {self}"
+            )
             if self._do_transfer:
                 try:
                     success = self._transfer(files_to_transfer)
@@ -186,6 +190,7 @@ class RSyncer(Observer):
                 outcome=TransferResult.SUCCESS,
                 transfer_total=self._files_transferred - previously_transferred,
                 queue_size=0,
+                base_path=self._basepath,
             )
             self.notify(update)
         return True
@@ -260,6 +265,7 @@ class RSyncer(Observer):
                     outcome=TransferResult.SUCCESS,
                     transfer_total=self._files_transferred - previously_transferred,
                     queue_size=current_outstanding,
+                    base_path=self._basepath,
                 )
                 if line[0] == ".":
                     # No transfer happening
@@ -274,7 +280,7 @@ class RSyncer(Observer):
                 return
 
         def parse_stderr(line: str):
-            logger.error(line)
+            logger.warning(line)
 
         relative_filenames = []
         for f in files:
@@ -283,20 +289,22 @@ class RSyncer(Observer):
             except ValueError:
                 raise ValueError(f"File '{f}' is outside of {self._basepath}") from None
         rsync_stdin = b"\n".join(os.fsencode(f) for f in relative_filenames)
+        rsync_cmd = [
+            "rsync",
+            "-iiv",
+            "--times",
+            "--progress",
+            "--outbuf=line",
+            "--files-from=-",
+            "-o",  # preserve ownership
+            "-p",  # preserve permissions
+        ]
+        if self._remove_files:
+            rsync_cmd.append("--remove-source-files")
+        rsync_cmd.extend([".", self._remote])
 
         result = procrunner.run(
-            [
-                "rsync",
-                "-iiv",
-                "--times",
-                "--progress",
-                "--outbuf=line",
-                "--files-from=-",
-                "-o",  # preserve ownership
-                "-p",  # preserve permissions
-                ".",
-                self._remote,
-            ],
+            rsync_cmd,
             callback_stdout=parse_stdout,
             callback_stderr=parse_stderr,
             working_directory=str(self._basepath),
