@@ -2,6 +2,8 @@ from __future__ import annotations
 
 import logging
 import os
+import queue
+import threading
 import time
 from pathlib import Path
 from typing import NamedTuple, Optional
@@ -23,6 +25,8 @@ class DirWatcher(murfey.util.Observer):
         self,
         path: str | os.PathLike,
         settling_time: float = 60,
+        appearance_time: float | None = None,
+        transfer_all: bool = True,
         status_bar: StatusBar | None = None,
     ):
         super().__init__()
@@ -31,12 +35,56 @@ class DirWatcher(murfey.util.Observer):
         self._file_candidates: dict[str, _FileInfo] = {}
         self._statusbar = status_bar
         self.settling_time = settling_time
+        self._appearance_time = appearance_time
+        self._transfer_all = transfer_all
         self._modification_overwrite: float | None = None
         self._init_time: float = time.time()
+        self.queue: queue.Queue[Path | None] = queue.Queue()
+        self.thread = threading.Thread(
+            name=f"DirWatcher {self._basepath}", target=self._process, daemon=True
+        )
+        self._stopping = False
+        self._halt_thread = False
 
     def __repr__(self) -> str:
         return f"<DirWatcher ({self._basepath})>"
 
+    def start(self):
+        if self.thread.is_alive():
+            raise RuntimeError("DirWatcher already running")
+        if self._stopping:
+            raise RuntimeError("DirWatcher has already stopped")
+        log.info(f"DirWatcher thread starting for {self}")
+        self.thread.start()
+
+    def stop(self):
+        log.debug("DirWatcher thread stop requested")
+        self._stopping = True
+        if self.thread.is_alive():
+            self.queue.join()
+
+        self._halt_thread = True
+        if self.thread.is_alive():
+            self.queue.put(None)
+            self.thread.join()
+        log.debug("DirWatcher thread stop completed")
+
+    def _process(self):
+        if self._appearance_time:
+            if self._appearance_time > 0:
+                modification_time: float | None = (
+                    time.time() - self._appearance_time * 3600
+                )
+            else:
+                modification_time = None
+        else:
+            modification_time = None
+        while not self._stopping:
+            self.scan(
+                modification_time=modification_time, transfer_all=self._transfer_all
+            )
+            time.sleep(15)
+
     def scan(self, modification_time: float | None = None, transfer_all: bool = False):
         try:
             filelist = self._scan_directory(
@@ -124,7 +172,8 @@ class DirWatcher(murfey.util.Observer):
                     self._statusbar.transferred[1] + 1,
                 ]
 
-        self.notify(Path(file_candidate))
+        if not Path(file_candidate).name.startswith("."):
+            self.notify(Path(file_candidate))
         del self._file_candidates[file_candidate]
 
     def _scan_directory(
