@@ -4,10 +4,10 @@ import argparse
 import logging
 import os
 import socket
-from functools import lru_cache, singledispatch
+from functools import lru_cache, partial, singledispatch
 from pathlib import Path
 from threading import Thread
-from typing import Any
+from typing import Any, List, NamedTuple
 
 import uvicorn
 import workflows
@@ -19,6 +19,7 @@ from ispyb.sqlalchemy._auto_db_schema import (
     DataCollection,
     DataCollectionGroup,
     ProcessingJob,
+    ProcessingJobParameter,
 )
 from rich.logging import RichHandler
 from sqlalchemy.exc import SQLAlchemyError
@@ -45,7 +46,12 @@ template_files = files("murfey") / "templates"
 templates = Jinja2Templates(directory=template_files)
 
 _running_server: uvicorn.Server | None = None
-_transport_object = None
+_transport_object: TransportManager | None = None
+
+
+class ExtendedRecord(NamedTuple):
+    record: Base
+    record_params: List[Base]
 
 
 def respond_with_template(filename: str, parameters: dict[str, Any] | None = None):
@@ -115,7 +121,10 @@ def run():
         "--feedback",
         action="store_true",
     )
-
+    parser.add_argument(
+        "--temporary",
+        action="store_true",
+    )
     verbosity = parser.add_mutually_exclusive_group()
     verbosity.add_argument(
         "-q",
@@ -165,10 +174,10 @@ def run():
     if murfey_machine_configuration:
         microscope = get_microscope()
         machine_config = from_file(Path(murfey_machine_configuration), microscope)
-
+    if not args.temporary and _transport_object:
+        _transport_object.feedback_queue = machine_config.feedback_queue
     rabbit_thread = Thread(
         target=feedback_listen,
-        kwargs={"feedback_queue": machine_config.feedback_queue},
         daemon=True,
     )
     logger.info("Starting Murfey RabbitMQ thread")
@@ -184,6 +193,8 @@ def run():
         host=args.host,
         port=args.port,
         log_config=None,
+        ws_ping_interval=300,
+        ws_ping_timeout=300,
     )
 
     _running_server = uvicorn.Server(config=config)
@@ -334,19 +345,37 @@ def feedback_callback(header: dict, message: dict) -> None:
             if dcgid is None:
                 _transport_object.transport.nack(header)
                 return None
-            global_state["data_collection_group_id"] = dcgid
-        _transport_object.transport.ack(header)
+            if global_state.get("data_collection_group_ids") and isinstance(
+                global_state["data_collection_group_ids"], dict
+            ):
+                global_state["data_collection_group_ids"] = {
+                    **global_state["data_collection_group_ids"],
+                    message.get("tag"): dcgid,
+                }
+            else:
+                global_state["data_collection_group_ids"] = {message.get("tag"): dcgid}
+            _transport_object.transport.ack(header)
         return None
     elif message["register"] == "data_collection":
+        dcgid = global_state.get("data_collection_group_ids", {}).get(  # type: ignore
+            message["tag"]
+        )
+        if dcgid is None:
+            raise ValueError(
+                f"No data collection group ID was found for image directory {message['image_directory']}"
+            )
         record = DataCollection(
             SESSIONID=message["session_id"],
             experimenttype=message["experiment_type"],
             imageDirectory=message["image_directory"],
             imageSuffix=message["image_suffix"],
             voltage=message["voltage"],
-            dataCollectionGroupId=global_state.get("data_collection_group_id"),
+            dataCollectionGroupId=dcgid,
+            pixelSizeOnImage=message["pixel_size"],
+            imageSizeX=message["image_size_x"],
+            imageSizeY=message["image_size_y"],
         )
-        dcid = _register(record, header)
+        dcid = _register(record, header, tag=message.get("tag"))
         if dcid is None and _transport_object:
             _transport_object.transport.nack(header)
             return None
@@ -367,7 +396,14 @@ def feedback_callback(header: dict, message: dict) -> None:
         assert isinstance(global_state["data_collection_ids"], dict)
         _dcid = global_state["data_collection_ids"][message["tag"]]
         record = ProcessingJob(dataCollectionId=_dcid, recipe=message["recipe"])
-        pid = _register(record, header)
+        if message.get("job_parameters"):
+            job_parameters = [
+                ProcessingJobParameter(parameterKey=k, parameterValue=v)
+                for k, v in message["job_parameters"].items()
+            ]
+            pid = _register(ExtendedRecord(record, job_parameters))
+        else:
+            pid = _register(record, header)
         if pid is None and _transport_object:
             _transport_object.transport.nack(header)
             return None
@@ -409,12 +445,12 @@ def feedback_callback(header: dict, message: dict) -> None:
 
 
 @singledispatch
-def _register(record, header: dict):
+def _register(record, header: dict, **kwargs):
     raise NotImplementedError(f"Not method to register {record} or type {type(record)}")
 
 
-@_register.register
-def _(record: Base, header: dict):
+@_register.register  # type: ignore
+def _(record: Base, header: dict, **kwargs):
     if not _transport_object:
         logger.error(
             f"No transport object found when processing record {record}. Message header: {header}"
@@ -422,7 +458,9 @@ def _(record: Base, header: dict):
         return None
     try:
         if isinstance(record, DataCollection):
-            return _transport_object.do_insert_data_collection(record)["return_value"]
+            return _transport_object.do_insert_data_collection(record, **kwargs)[
+                "return_value"
+            ]
         if isinstance(record, DataCollectionGroup):
             return _transport_object.do_insert_data_collection_group(record)[
                 "return_value"
@@ -449,9 +487,34 @@ def _(record: Base, header: dict):
         )
         return None
 
+@_register.register  # type: ignore
+def _(extended_record: ExtendedRecord, header: dict, **kwargs):
+    return _transport_object.do_create_ispyb_job(
+        extended_record.record, params=extended_record.record_params
+    )["return_value"]
 
-def feedback_listen(feedback_queue: str = "murfey_feedback"):
+
+@_register.register  # type: ignore
+def _(extended_record: ExtendedRecord, header: dict, **kwargs):
+    return _transport_object.do_create_ispyb_job(
+        extended_record.record, params=extended_record.record_params
+    )["return_value"]
+
+
+def feedback_listen():
     if _transport_object:
+        if not _transport_object.feedback_queue:
+            _transport_object.feedback_queue = (
+                _transport_object.transport._subscribe_temporary(
+                    channel_hint="", callback=None, sub_id=None
+                )
+            )
+        _transport_object._connection_callback = partial(
+            _transport_object.transport.subscribe,
+            _transport_object.feedback_queue,
+            feedback_callback,
+            acknowledgement=True,
+        )
         _transport_object.transport.subscribe(
-            feedback_queue, feedback_callback, acknowledgement=True
+            _transport_object.feedback_queue, feedback_callback, acknowledgement=True
         )
