@@ -9,13 +9,18 @@ from pydantic import BaseModel
 
 class MachineConfig(BaseModel):
     acquisition_software: List[str]
-    calibrations: Dict[str, Union[dict, float]]
+    calibrations: Dict[str, Dict[str, Union[dict, float]]]
     data_directories: Dict[Path, str]
     rsync_basepath: Path
+    software_versions: Dict[str, str] = {}
+    external_executables: Dict[str, str] = {}
     rsync_module: str = ""
     gain_reference_directory: Optional[Path] = None
     processed_directory_name: str = "processed"
     feedback_queue: str = "murfey_feedback"
+    superres: bool = False
+    camera: str = "FALCON"
+    data_required_substrings: Dict[str, List[str]] = {}
 
 
 def from_file(config_file_path: Path, microscope: str) -> MachineConfig:
