@@ -2,7 +2,7 @@ from __future__ import annotations
 
 from datetime import datetime
 from pathlib import Path
-from typing import Any, Dict, Optional
+from typing import Any, Dict, List, Optional
 
 from pydantic import BaseModel
 
@@ -40,6 +40,10 @@ class File(BaseModel):
     timestamp: float
 
 
+class SPAProcessingParameters(BaseModel):
+    job_id: int
+
+
 class ProcessFile(BaseModel):
     path: str
     description: str
@@ -51,6 +55,9 @@ class ProcessFile(BaseModel):
     mc_uuid: int
     autoproc_program_id: int
     pixel_size: float
+    dose_per_frame: float
+    mc_binning: int = 1
+    gain_ref: Optional[str] = None
 
 
 class TiltSeries(BaseModel):
@@ -61,15 +68,19 @@ class TiltSeries(BaseModel):
     autoproc_program_id: int
     motion_corrected_path: str
     movie_id: int
+    pixel_size: float
+    manual_tilt_offset: int
 
 
 class SuggestedPathParameters(BaseModel):
     base_path: Path
+    touch: bool = False
 
 
 class DCGroupParameters(BaseModel):
     experiment_type: str
     experiment_type_id: int
+    tag: str
 
 
 class DCParameters(BaseModel):
@@ -78,18 +89,61 @@ class DCParameters(BaseModel):
     experiment_type: str
     image_size_x: int
     image_size_y: int
-    tilt: int
     file_extension: str
     acquisition_software: str
     image_directory: str
     tag: str
 
 
+class DCParametersTomo(BaseModel):
+    dose_per_frame: float
+    gain_ref: Optional[str]
+    experiment_type: str
+    voltage: float
+    image_size_x: int
+    image_size_y: int
+    pixel_size_on_image: str
+    motion_corr_binning: int
+    manual_tilt_offset: float
+    file_extension: str
+    acquisition_software: str
+
+
+class DCParametersSPA(BaseModel):
+    dose_per_frame: float
+    gain_ref: Optional[str]
+    experiment_type: str
+    voltage: float
+    image_size_x: int
+    image_size_y: int
+    pixel_size_on_image: str
+    motion_corr_binning: int
+    file_extension: str
+    acquisition_software: str
+    use_cryolo: bool
+    symmetry: str
+    mask_diameter: int
+    boxsize: int
+    downscale: bool
+    small_boxsize: int
+    eer_grouping: int
+
+
 class ProcessingJobParameters(BaseModel):
     tag: str
     recipe: str
+    parameters: Dict[str, Any] = {}
 
 
 class RegistrationMessage(BaseModel):
     registration: str
     params: Optional[Dict[str, Any]] = None
+
+
+class ConnectionFileParameters(BaseModel):
+    filename: str
+    destinations: List[str]
+
+
+class GainReference(BaseModel):
+    gain_ref: Path
